# Autogenerated by nbdev

d = { 'settings': { 'branch': 'main',
                'doc_baseurl': '/LatePenalty',
                'doc_host': 'https://scott-yj-yang.github.io',
                'git_url': 'https://github.com/scott-yj-yang/LatePenalty',
                'lib_path': 'LatePenalty'},
  'syms': { 'LatePenalty.gradescope': { 'LatePenalty.gradescope.bcolors': ( 'api/gradescope_process_grade.html#bcolors',
                                                                            'LatePenalty/gradescope.py'),
                                        'LatePenalty.gradescope.gradescope_grade': ( 'api/gradescope_process_grade.html#gradescope_grade',
                                                                                     'LatePenalty/gradescope.py'),
                                        'LatePenalty.gradescope.gradescope_grade.__init__': ( 'api/gradescope_process_grade.html#gradescope_grade.__init__',
                                                                                              'LatePenalty/gradescope.py'),
                                        'LatePenalty.gradescope.gradescope_grade._post_grade': ( 'api/gradescope_process_grade.html#gradescope_grade._post_grade',
                                                                                                 'LatePenalty/gradescope.py'),
                                        'LatePenalty.gradescope.gradescope_grade.auth_canvas': ( 'api/gradescope_process_grade.html#gradescope_grade.auth_canvas',
                                                                                                 'LatePenalty/gradescope.py'),
                                        'LatePenalty.gradescope.gradescope_grade.calculate_credit_balance': ( 'api/gradescope_process_grade.html#gradescope_grade.calculate_credit_balance',
                                                                                                              'LatePenalty/gradescope.py'),
                                        'LatePenalty.gradescope.gradescope_grade.calculate_late_hour': ( 'api/gradescope_process_grade.html#gradescope_grade.calculate_late_hour',
                                                                                                         'LatePenalty/gradescope.py'),
                                        'LatePenalty.gradescope.gradescope_grade.calculate_total_score': ( 'api/gradescope_process_grade.html#gradescope_grade.calculate_total_score',
                                                                                                           'LatePenalty/gradescope.py'),
                                        'LatePenalty.gradescope.gradescope_grade.link_assignment': ( 'api/gradescope_process_grade.html#gradescope_grade.link_assignment',
                                                                                                     'LatePenalty/gradescope.py'),
                                        'LatePenalty.gradescope.gradescope_grade.load_gradescope_csv': ( 'api/gradescope_process_grade.html#gradescope_grade.load_gradescope_csv',
                                                                                                         'LatePenalty/gradescope.py'),
                                        'LatePenalty.gradescope.gradescope_grade.post_to_canvas': ( 'api/gradescope_process_grade.html#gradescope_grade.post_to_canvas',
                                                                                                    'LatePenalty/gradescope.py'),
                                        'LatePenalty.gradescope.gradescope_grade.set_course': ( 'api/gradescope_process_grade.html#gradescope_grade.set_course',
                                                                                                'LatePenalty/gradescope.py')},
            'LatePenalty.nbgrader': { 'LatePenalty.nbgrader.bcolors': ( 'api/nbgrader_process_grade.html#bcolors',
                                                                        'LatePenalty/nbgrader.py'),
                                      'LatePenalty.nbgrader.nbgrader_grade': ( 'api/nbgrader_process_grade.html#nbgrader_grade',
                                                                               'LatePenalty/nbgrader.py'),
                                      'LatePenalty.nbgrader.nbgrader_grade.__init__': ( 'api/nbgrader_process_grade.html#nbgrader_grade.__init__',
                                                                                        'LatePenalty/nbgrader.py'),
                                      'LatePenalty.nbgrader.nbgrader_grade._calculate_late_days': ( 'api/nbgrader_process_grade.html#nbgrader_grade._calculate_late_days',
                                                                                                    'LatePenalty/nbgrader.py'),
                                      'LatePenalty.nbgrader.nbgrader_grade._parse_assignments': ( 'api/nbgrader_process_grade.html#nbgrader_grade._parse_assignments',
                                                                                                  'LatePenalty/nbgrader.py'),
                                      'LatePenalty.nbgrader.nbgrader_grade._post_grade': ( 'api/nbgrader_process_grade.html#nbgrader_grade._post_grade',
                                                                                           'LatePenalty/nbgrader.py'),
                                      'LatePenalty.nbgrader.nbgrader_grade.auth_canvas': ( 'api/nbgrader_process_grade.html#nbgrader_grade.auth_canvas',
                                                                                           'LatePenalty/nbgrader.py'),
                                      'LatePenalty.nbgrader.nbgrader_grade.calculate_credit_balance': ( 'api/nbgrader_process_grade.html#nbgrader_grade.calculate_credit_balance',
                                                                                                        'LatePenalty/nbgrader.py'),
                                      'LatePenalty.nbgrader.nbgrader_grade.get_late_days': ( 'api/nbgrader_process_grade.html#nbgrader_grade.get_late_days',
                                                                                             'LatePenalty/nbgrader.py'),
                                      'LatePenalty.nbgrader.nbgrader_grade.link_assignment': ( 'api/nbgrader_process_grade.html#nbgrader_grade.link_assignment',
                                                                                               'LatePenalty/nbgrader.py'),
                                      'LatePenalty.nbgrader.nbgrader_grade.load_grades_csv': ( 'api/nbgrader_process_grade.html#nbgrader_grade.load_grades_csv',
                                                                                               'LatePenalty/nbgrader.py'),
                                      'LatePenalty.nbgrader.nbgrader_grade.load_late_exception': ( 'api/nbgrader_process_grade.html#nbgrader_grade.load_late_exception',
                                                                                                   'LatePenalty/nbgrader.py'),
                                      'LatePenalty.nbgrader.nbgrader_grade.post_to_canvas': ( 'api/nbgrader_process_grade.html#nbgrader_grade.post_to_canvas',
                                                                                              'LatePenalty/nbgrader.py'),
                                      'LatePenalty.nbgrader.nbgrader_grade.set_course': ( 'api/nbgrader_process_grade.html#nbgrader_grade.set_course',
                                                                                          'LatePenalty/nbgrader.py')}}}
